(define-module (ui mkve ajax)
               :use-module (alterator ajax)
               :use-module (alterator woo)

               :export (init))

(define (say message) (format #t (string-append message "~%")))

(define (set-autostart)
  (woo-write "/mkve/autostart"
             'apply "yes"
             'name (form-value "name")
             'autostart (form-value "autostart")))

(define (autostart-changed)
  (if (not (catch/message set-autostart)) (update-autostart)))

(define (redirect-to-create) (form-replace "/mkve/create"))

(define (redirect-to-configure)
  (form-replace
    (string-append
      "/mkve/"
      (form-value "hidden_hypervisor")
      "?machine="
      (form-value "name")
      )))

(define (update-pluck-visibility pluck)
    (form-update-visibility "pluck" (not (string=? "" pluck))))

(define (update-button-start-visibility state)
  (cond
    ((string=? state "5") (form-update-activity "start" #t))
    ((string=? state "6") (form-update-activity "start" #t))
    (else (form-update-activity "start" #f))))

(define (update-button-stop-visibility state)
  (cond
    ((string=? state "1") (form-update-activity "stop" #t))
    ((string=? state "2") (form-update-activity "stop" #t))
    ((string=? state "3") (form-update-activity "stop" #t))
    (else (form-update-activity "stop" #f))))

(define (update-button-restart-visibility state)
  (cond
    ((string=? state "1") (form-update-activity "restart" #t))
    ((string=? state "2") (form-update-activity "restart" #t))
    ((string=? state "3") (form-update-activity "restart" #t))
    ((string=? state "4") (form-update-activity "restart" #t))
    (else (form-update-activity "restart" #f))))

(define (update-buttons-activity state)
  (update-button-start-visibility state)
  (update-button-stop-visibility state)
  (update-button-restart-visibility state))

(define (list_ip hypervisor)
  (woo-get-option
    (woo-read-first
      (string-append "/mkve-" hypervisor "/ip")
                     'machine (form-value "name") ; the 'name is known as 'machine inside hypervisor-specific backends
                     'language (form-value "language"))
    'ip
    )
  )

(define (update-read)
    (let
      ((data (woo-read-first "/mkve" 'name (form-value "name") 'language (form-value "language"))))
        (form-update-value "autostart" (woo-get-option data 'autostart))
        (form-update-value "hypervisor" (woo-get-option data 'hypervisor))
        (form-update-value "hidden_hypervisor" (woo-get-option data 'hypervisor))
        (form-update-value "state" (woo-get-option data 'state))
        (form-update-value "hidden_state" (woo-get-option data 'hidden_state))
        (form-update-value "cpu_time" (woo-get-option data 'cpu_time))
        (form-update-value "ip" (list_ip (woo-get-option data 'hypervisor)))
        (update-buttons-activity (woo-get-option data 'hidden_state))
        (update-pluck-visibility (woo-get-option data 'hidden_pluck))))

(define (update-machines)
  (let
    ((machines (woo-list "/mkve/list_machines")))
    (form-update-enum "name" machines)
    (form-update-value "name" (woo-get-option (woo-read-first "/mkve") 'name))))

(define (machine action)
  (catch/message
    (lambda ()
      (woo-write "/mkve"
                 'apply "yes"
                 'name (form-value "name")
                 'machine_action action)))
  (update-read)
  (if (string=? action "destroy") (update-machines)))

(define (start-machine) (machine "start"))
(define (stop-machine) (machine "stop"))
(define (restart-machine) (machine "restart"))
(define (destroy-machine) (machine "destroy"))

(define (init)
  (update-buttons-activity (form-value "hidden_state"))

  ; the following is temporary hack. Must use wf=none otherwise
  (form-update-value "ip" (list_ip (form-value "hidden_hypervisor")))

  (form-bind "name" "change" update-read)
  (form-bind "autostart" "change" autostart-changed)

  (form-bind "create" "click" redirect-to-create)
  (form-bind "configure" "click" redirect-to-configure)
  (form-bind "start" "click" start-machine)
  (form-bind "stop" "click" stop-machine)
  (form-bind "restart" "click" restart-machine)
  (form-bind "destroy" "click" destroy-machine)
)
