(define-module (ui mkve create ajax)
               :use-module (alterator ajax)
               :use-module (alterator woo)

               :export (init))

(define (say message) (format #t (string-append message "~%")))

(define (setup-openvz)
  (form-update-visibility "openvz" #t)
  (form-update-visibility "kvm" #f)
  )
(define (setup-kvm kvm_bridge)
  (form-update-visibility "openvz" #f)
  (form-update-visibility "kvm" #t)
  (form-update-enum "kvm_bridge" (woo-list "/mkve-create/bridges"))
  (form-update-value "kvm_bridge" kvm_bridge)
  )

(define (setup-bundle bundle)
  (let*
    ((data (woo-read-first "/mkve-create" 'bundle bundle))
     (hypervisor (woo-get-option data 'hypervisor))
     (vendor (woo-get-option data 'vendor))
     (arch (woo-get-option data 'arch))
     (kvm_bridge (woo-get-option data 'kvm_bridge))
     (kvm_memory (woo-get-option data 'kvm_memory))
     )
    (form-update-value "fullpath" bundle)
    (form-update-value "hypervisor" hypervisor)
    (form-update-value "vendor" vendor)
    (form-update-value "arch" arch)
    (form-update-value "kvm_memory" kvm_memory)

    (if (string=? "openvz" hypervisor)
      (setup-openvz)
      (setup-kvm kvm_bridge)
      )
    )
  )

(define (setup-form)
  (form-update-enum "bundle" (woo-list "/mkve-create/bundles"))
  (let
    ((bundle (woo-get-option (woo-read-first "/mkve-create/default_bundle") 'bundle)))
    (form-update-value "bundle" bundle)
    (setup-bundle bundle)
    )
  )

(define (machine_name)
  (form-value (string-append (form-value "hypervisor") "_machine_name")))

(define (do-create)
  (woo-write "/mkve-create"
             'create "yes"
             'machine_name (machine_name)
             'bundle (form-value "bundle")
             'kvm_bridge (form-value "kvm_bridge")
             'kvm_memory (form-value "kvm_memory")
             )
  )

(define (create)
  (if (catch/message do-create)
    (form-replace
      (string-append
        "/mkve/" (form-value "hypervisor") "?machine=" (machine_name)))))

(define (bundle-changed) (setup-bundle (form-value "bundle")))

(define (do-search)
  (woo-write "/mkve-create" 'cache "yes")

  (form-update-visibility "bundles" #t)
  (form-update-visibility "no_bundles" #f)

  (setup-form)
  )

(define (search) (catch/message do-search))

(define (init)
  (form-update-visibility "bundles" #f)

  (form-bind "bundle" "change" bundle-changed)

  (form-bind "search" "click" search)
  (form-bind "update" "click" search)
  (form-bind "create" "click" create)
  )
